/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "DistrhoUI.hpp"
#include "ResizeHandle.hpp"

START_NAMESPACE_DISTRHO

// --------------------------------------------------------------------------------------------------------------------
enum HeavyParams {CRSHR,FLDR,GAIN,LMTR,MIX,SMTHR,SQNC,
};

class ImGuiPluginUI : public UI
{
    
    int fcrshr = 512.0;
    float ffldr = 1.0f;
    float fgain = 0.0f;
    bool flmtr = 1.0f != 0.0f;
    float fmix = 50.0f;
    float fsmthr = 1.0f;
    int fsqnc = 0.0;

    ResizeHandle fResizeHandle;

    // ----------------------------------------------------------------------------------------------------------------

public:
   /**
      UI class constructor.
      The UI should be initialized to a default state that matches the plugin side.
    */
    ImGuiPluginUI()
        : UI(DISTRHO_UI_DEFAULT_WIDTH, DISTRHO_UI_DEFAULT_HEIGHT),
          fResizeHandle(this)
    {
        setGeometryConstraints(DISTRHO_UI_DEFAULT_WIDTH, DISTRHO_UI_DEFAULT_HEIGHT, true);

        // hide handle if UI is resizable
        if (isResizable())
            fResizeHandle.hide();
    }

protected:
    // ----------------------------------------------------------------------------------------------------------------
    // DSP/Plugin Callbacks

   /**
      A parameter has changed on the plugin side.@n
      This is called by the host to inform the UI about parameter changes.
    */
    void parameterChanged(uint32_t index, float value) override
    {
        switch (index) {
            case CRSHR:
                fcrshr = value;
                break;
            case FLDR:
                ffldr = value;
                break;
            case GAIN:
                fgain = value;
                break;
            case LMTR:
                flmtr = value != 0.0f;
                break;
            case MIX:
                fmix = value;
                break;
            case SMTHR:
                fsmthr = value;
                break;
            case SQNC:
                fsqnc = value;
                break;
            
            default: return;
        }
    
        repaint();
    }

    // ----------------------------------------------------------------------------------------------------------------
    // Widget Callbacks

   /**
      ImGui specific onDisplay function.
    */
    void onImGuiDisplay() override
    {
        const float width = getWidth();
        const float height = getHeight();
        const float margin = 20.0f * getScaleFactor();

        ImGui::SetNextWindowPos(ImVec2(margin, margin));
        ImGui::SetNextWindowSize(ImVec2(width - 2 * margin, height - 2 * margin));

        if (ImGui::Begin("WSTD MANGLR", nullptr, ImGuiWindowFlags_NoResize + ImGuiWindowFlags_NoCollapse))
        {
            if (ImGui::SliderInt("Crshr", &fcrshr, 2.0f, 512.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(CRSHR, true);
                }
                setParameterValue(CRSHR, fcrshr);
            }
    
            if (ImGui::SliderFloat("Fldr", &ffldr, 1.0f, 13.37f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(FLDR, true);
                }
                setParameterValue(FLDR, ffldr);
            }
    
            if (ImGui::SliderFloat("Gain", &fgain, -20.0f, 0.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(GAIN, true);
                }
                setParameterValue(GAIN, fgain);
            }
    
            if (ImGui::Toggle("Lmtr", &flmtr))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(LMTR, true);
                }
                setParameterValue(LMTR, flmtr);
            }
    
            if (ImGui::SliderFloat("Mix", &fmix, 0.0f, 100.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(MIX, true);
                }
                setParameterValue(MIX, fmix);
            }
    
            if (ImGui::SliderFloat("Smthr", &fsmthr, 1.0f, 13.37f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(SMTHR, true);
                }
                setParameterValue(SMTHR, fsmthr);
            }
    

            const char* sqnc_list[6] = {
                "C~F~S",
                "C~S~F",
                "F~C~S",
                "F~S~C",
                "S~C~F",
                "S~F~C",
            };

            if (ImGui::BeginCombo("Sqnc", sqnc_list[fsqnc]))
            {
                for (int n = 0; n < 6; n++)
                {
                    bool is_selected = (fsqnc == n);
                    if (ImGui::Selectable(sqnc_list[n], is_selected))
                    {
                        fsqnc = n;
                        editParameter(SQNC, true);
                        setParameterValue(SQNC, fsqnc);
                    }
                    if (is_selected)
                        ImGui::SetItemDefaultFocus();
                }
                ImGui::EndCombo();
            }
    
            if (ImGui::IsItemDeactivated())
            {
            editParameter(CRSHR, false);
            editParameter(FLDR, false);
            editParameter(GAIN, false);
            editParameter(LMTR, false);
            editParameter(MIX, false);
            editParameter(SMTHR, false);
            editParameter(SQNC, false);
            }
        }
        ImGui::End();
    }

    DISTRHO_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(ImGuiPluginUI)
};

// --------------------------------------------------------------------------------------------------------------------

UI* createUI()
{
    return new ImGuiPluginUI();
}

// --------------------------------------------------------------------------------------------------------------------

END_NAMESPACE_DISTRHO